#ifndef __EVENT_HPP__
#define __EVENT_HPP__

#ifndef __cplusplus
#error Event requires C++ compilation
#endif // !defined __EVENT_HPP__

#include <pshpack8.h>

class EventArgs
{
public:
    // Construction/Destruction
    EventArgs() throw();
    virtual ~EventArgs() throw();
};

class IObject
{
public:
    virtual ~IObject() throw();
};

typedef void (__stdcall IObject::*EventHandler)(IObject * pSender,EventArgs * pArgs);
typedef void (__stdcall * GlobalEventHandler)(IObject * pSender,EventArgs * pArgs);

class IDelegate
{
public:
    virtual ~IDelegate() throw();
    // Interface Members
    virtual void Invoke() throw() = 0;
    virtual void SetArguments(IObject * pSender,EventArgs * pArgs) throw() = 0;
    virtual IObject * GetObject() const throw() = 0;
    virtual EventHandler GetCallback() const throw() = 0;
};

class Delegate : public IDelegate
{
public:
    // Construction/Destruction
    Delegate(IObject * pObject,EventHandler pfnCallback) throw();
    Delegate(GlobalEventHandler pfnCallback) throw();
    virtual ~Delegate() throw();
    // IDelegate
    virtual void Invoke() throw();
    virtual void SetArguments(IObject * pSender,EventArgs * pArgs) throw();
    virtual IObject * GetObject() const throw();
    virtual EventHandler GetCallback() const throw();
protected:
    // Member Variables
    IObject * Object;
    union
    {
        EventHandler Callback;
        GlobalEventHandler GlobalCallback;
    };
    IObject * Sender;
    EventArgs * Args;
};

class Event
{
public:
    // Construction/Destruction
    Event() throw();
    virtual ~Event() throw();
    // Member Functions
    bool AddHandler(IDelegate * pDelegate) throw();
    bool RemoveHandler(IDelegate * pDelegate) throw();
    void RemoveAllHandlers() throw();
    virtual void Raise(IObject * pSender,EventArgs * pArgs) throw();
protected:
    // Member Variables
    IDelegate ** EventHandlers;
    unsigned long Capacity;
    unsigned long Length;
    unsigned long BlockSize;
    // Member Functions
    unsigned long DetermineCapacity(unsigned long dwElemCount) throw();
};

#include <poppack.h>

#endif // !defined __EVENT_HPP__
